#!/usr/bin/perl
use DBI;
use strict;

#locking routine from camel book
sub LOCK_SH { 1 }
sub LOCK_EX { 2 }
sub LOCK_NB { 4 }
sub LOCK_UN { 8 }

sub SECS_WK { 604800 }
sub SECS_DAY { 86400 }
sub SECS_HR { 3600 }
sub SECS_MIN { 60 }


my $database = "mp3stat";
my $user = "";
my $password = "";

open PASSWD, "<$ENV{'HOME'}/.mp3sqlrc";

while (my $l=<PASSWD>) {
    chomp $l;
    eval "$l";
}


my  $dbh = DBI->connect("dbi:mysql:database=mp3stat", $user, $password, {AutoCommit => 1})
    || die $DBI::errstr;


my $maxlines=-1;
my @summaries;
my $quiet=1;

my $now = time();
# by default a summary queries the last week.
my ($since,$before) = ($now-7*SECS_DAY,$now);

my $i;
for ($i=0; $i<=$#ARGV;) {
    if ($ARGV[$i] =~ /--summary/) {
	doSummary();
	splice(@ARGV, $i, 1);
    } elsif ($ARGV[$i] =~ /--push/) {
	pushFile($ARGV[$i+1]);
	splice(@ARGV, $i, 2);
    } elsif ($ARGV[$i] =~ /--maxlines/) {
	$maxlines=$ARGV[$i+1];
	splice(@ARGV, $i, 2);
    } elsif ($ARGV[$i] =~ /--since/) {
	$since=adjusttime($ARGV[$i+1]);
	splice(@ARGV, $i, 2);
    } elsif ($ARGV[$i] =~ /--before/) {
	$before=adjusttime($ARGV[$i+1]);
	splice(@ARGV, $i, 2);
    } elsif ($ARGV[$i] =~ /--compare/) {
	compareSummaries();
	splice(@ARGV, $i, 1);
    } elsif ($ARGV[$i] =~ /--print/) {
	printSummary();
	splice(@ARGV, $i, 1);
    } elsif ($ARGV[$i] =~ /--export/) {
	exportDB();
	splice(@ARGV, $i, 1);
    } else {
	$i++;
    }
}
print STDERR "Unprocessed args: ", join ' ', @ARGV, "\n" if ($#ARGV>=0);

$dbh->disconnect();
exit;

sub pushFile() {
    my $name=shift;
    $dbh->do("INSERT INTO testmp3 VALUES ( ".$now.",".$dbh->quote($name).")");
    print STDERR "Pushed: $now,$name\n" unless $quiet;
}


sub printSummary() {
    my $s;
    my $lines=0;

    my $summary=pop @summaries;
    foreach $s (@{$summary}) {
	print "$s\n" if ($lines<$maxlines || $maxlines<0);
	$lines++;
    }
}

sub compareSummaries() {
    my @newsummary;
    my $thistime=pop @summaries;
    my $lasttime=pop @summaries;

    my ($line1, $line2);
    foreach $line1 (@{$thistime}) {
	my $posn=1;
	my $found=0;
	my ($key1)=$line1=~/^(.*)\t/;
	foreach $line2 (@{$lasttime}) {
	    my ($key2)=$line2=~/^(.*)\t/;
	    if ($key1 eq $key2) {
		$line1.="\t$posn";
		$found=1;
	    }
	    $posn++;
	}
#	print STDERR "Found: $found\n";
	$line1.="\t--" if (!$found);
	push @newsummary, $line1;
    }
    push @summaries, [ @newsummary ];
}

sub adjusttime() {
    my ($adjustment)=@_;
    $adjustment =~ /(.*)w/ and $adjustment=$now-$1*SECS_WK;
    $adjustment =~ /(.*)d/ and $adjustment=$now-$1*SECS_DAY;
    $adjustment =~ /(.*)m/ and $adjustment=$now-$1*SECS_MIN;
    $adjustment =~ /(.*)h/ and $adjustment=$now-$1*SECS_HR;
    $adjustment =~ /(.*)s/ and $adjustment=$now-$1;
    return $adjustment;
}


sub doSummary() {
    my @summary;
    my @tmpsumm;
    my @stmpsumm;
    my $key;
    print STDERR "Now: ", scalar localtime($now),
	" Since: ", scalar localtime($since),
	    " Before: ", scalar localtime($before), "\n" unless $quiet;

    
    my $sth = $dbh->prepare("SELECT COUNT(*) as count,name,MAX(timestamp) as mts FROM testmp3 WHERE timestamp BETWEEN $since AND $before ".
			    " GROUP BY name  ORDER BY count DESC, mts DESC");
    $sth->execute();
    while (my $ref = $sth->fetchrow_hashref()) {
	print "Found a row: id = $ref->{'count'}, name = $ref->{'name'}\n" unless $quiet;
	next if $ref->{'name'} =~ /null/;
	next if $ref->{'name'} =~ /CD Audio/;
	push @summary, "$ref->{'name'}\t$ref->{'count'}";
    }
    $sth->finish();
    push @summaries, [@summary];
}

sub totalcmp { 
    my @ta = split /:/, $a;
    my @tb = split /:/, $b;
# comparing the last elems gives the more recent the preference
    $ta[1] <=> $tb[1] or $ta[2] <=> $tb[2];	    
}


sub exportDB {
    my $sth = $dbh->prepare("SELECT * from testmp3");
    $sth->execute();
    while (my $ref = $sth->fetchrow_hashref()) {
	print "$ref->{'timestamp'}\t$ref->{'name'}\n";
    }
}

